@@ -4,8 +4,8 @@
 
 #include <glib-object.h>
 
-#include "gsd-smartcard-manager.h"
-/* enumerations from "gsd-smartcard-manager.h" */
+#include "/home/hadess/Projects/jhbuild/gnome-settings-daemon/plugins/smartcard/gsd-smartcard-manager.h"
+/* enumerations from "/home/hadess/Projects/jhbuild/gnome-settings-daemon/plugins/smartcard/gsd-smartcard-manager.h" */
 GType gsd_smartcard_manager_error_get_type (void) G_GNUC_CONST;
 
 GType
